NdArray

Struct NdArray 

pub struct NdArray<'a, T: RawDataType> { /* private fields */ }

Implementations

Source

impl<'a, T: RawDataType> NdArray<'a, T>

pub fn value(&self) -> T

Retrieves the single value contained within an ndarray with a singular element.

Panics

If the ndarray contains more than one element (i.e., it is not a scalar or an ndarray with a single element)

Example

let ndarray = NdArray::scalar(50f32);
let value = ndarray.value();
assert_eq!(value, 50.0);
Notes

This function is only meant for arrays that are guaranteed to have exactly one element. For arrays with multiple elements, consider using appropriate methods to access individual elements or slices safely.

pub fn data_slice(&self) -> &'a [T]

Returns a slice of the ndarray’s (flattened) data buffer

Example

let ndarray = NdArray::new([[50, 60], [-5, -10]]);
let data = ndarray.data_slice();
assert_eq!(data, &[50, 60, -5, -10]);
pub fn into_data_vector(self) -> Vec<T>

Converts an NdArray into its underlying data vector by flattening its dimensions.

Panics
  • If the ndarray does not own its data (it is a NdArray view).
Example

let ndarray = NdArray::new([[50, 60], [-5, -10]]);
let data = ndarray.into_data_vector();
assert_eq!(data, vec![50, 60, -5, -10]);
Source

impl<T: RawDataType> NdArray<'_, T>

pub fn flatiter(&self) -> FlatIterator<T>

pub fn flatiter_ptr(&self) -> BufferIterator<T>

Source

impl<'a, T: RawDataType> NdArray<'a, T>

pub fn iter(&'a self) -> NdIterator<'a, T>

pub fn iter_along(&'a self, axis: impl AxisType) -> NdIterator<'a, T>

pub fn nditer(&'a self, axes: impl AxesType) -> NdIterator<'a, T>

Source

impl<'a, T: RawDataType> NdArray<'a, T>

pub fn flatten(&self) -> NdArray<'static, T>

Returns a 1D copy of a flattened multidimensional ndarray.

If copying the data is not desirable, it may be possible to return a view. See NdArray::ravel().

Examples

let ndarray = NdArray::new([[1, 2, 3], [4, 5, 6]]);
let flat_array = ndarray.flatten();
assert_eq!(flat_array, NdArray::new([1, 2, 3, 4, 5, 6]));
Source

impl<T: NumericDataType> NdArray<'_, T>

pub fn sum(&self) -> NdArray<'static, T>

Computes the sum of all elements in the array.

Example
use redstone_ml::*;

let array = NdArray::new(vec![1, 2, 3, 4]);
let sum = array.sum();
assert_eq!(sum.value(), 1 + 2 + 3 + 4);
pub fn sum_along(&self, axes: impl ToVec<isize>) -> NdArray<'static, T>

pub fn product(&self) -> NdArray<'static, T>

Computes the product of all elements in the array.

Example
use redstone_ml::*;

let array = NdArray::new(vec![1, 2, 3, 4]);
let prod = array.product();
assert_eq!(prod.value(), 1 * 2 * 3 * 4);
pub fn product_along(&self, axes: impl ToVec<isize>) -> NdArray<'static, T>

pub fn min(&self) -> NdArray<'static, T>

Computes the minimum of all elements in the array.

Example
use redstone_ml::*;

let array = NdArray::new(vec![-1, 3, -7, 8]);
let min = array.min();
assert_eq!(min.value(), -7);
pub fn min_along(&self, axes: impl ToVec<isize>) -> NdArray<'static, T>

pub fn max(&self) -> NdArray<'static, T>

Computes the maximum of all elements in the array.

Example
use redstone_ml::*;

let array = NdArray::new(vec![-1, 3, -7, 8]);
let max = array.max();
assert_eq!(max.value(), 8);
pub fn max_along(&self, axes: impl ToVec<isize>) -> NdArray<'static, T>

pub fn min_magnitude(&self) -> NdArray<'static, T>

Computes the minimum absolute value of all elements in the array.

Example
use redstone_ml::*;

let array = NdArray::new(vec![-1, 3, -7, 8]);
let min = array.min_magnitude();
assert_eq!(min.value(), 1);
pub fn min_magnitude_along( &self, axes: impl ToVec<isize>, ) -> NdArray<'static, T>

pub fn max_magnitude(&self) -> NdArray<'static, T>

Computes the maximum absolute value of all elements in the array.

Example
use redstone_ml::*;

let array = NdArray::new(vec![-1, 3, -9, 8]);
let max = array.max_magnitude();
assert_eq!(max.value(), 9);
pub fn max_magnitude_along( &self, axes: impl ToVec<isize>, ) -> NdArray<'static, T>

pub fn mean(&self) -> NdArray<'static, T>
where T: FloatDataType,

Computes the mean of all elements in the array.

Example
use redstone_ml::*;

let array = NdArray::new(vec![1.0, 3.0, 5.0, 7.0]);
let mean = array.mean();
assert_eq!(mean.value(), 4.0);
pub fn mean_along(&self, axes: impl ToVec<isize>) -> NdArray<'static, T>
where T: FloatDataType,

Source

impl<'a, T: RawDataType> NdArray<'a, T>

pub fn slice_along<S: Indexer>(&'a self, axis: Axis, index: S) -> NdArray<'a, T>

pub fn slice<S, I>(&'a self, index: I) -> NdArray<'a, T>
where S: Indexer, I: IntoIterator<Item = S>,

Source

impl<T: RawDataType> NdArray<'_, T>

pub fn fill(&mut self, value: T)

Fills the entire array with a specified value.

Example

let mut arr = NdArray::new([1, 2, 4]);
arr.fill(10);
assert_eq!(arr, NdArray::new([10, 10, 10]));
Source

impl<T: RawDataType + From<bool>> NdArray<'_, T>

pub fn zero(&mut self)

Fills the entire array with a zero (or false if dtype is boolean).

Example

let mut arr = NdArray::new([1, 2, 4]);
arr.zero();
assert_eq!(arr, NdArray::new([0, 0, 0]));
Source

impl<'a, T: RawDataType> NdArray<'a, T>

pub fn clone<'r>(&self) -> NdArray<'r, T>

Source

impl<'a, T: RawDataType> NdArray<'a, T>

pub fn broadcast_to(&'a self, shape: &[usize]) -> NdArray<'a, T>

Broadcasts the NdArray to the specified shape.

This method returns a readonly view of the ndarray with the desired shape. Broadcasting is done by left-padding the ndarray’s shape with ones until they reach the desired dimension. Then, any axes with length 1 are repeated to match the target shape.

For example, suppose the ndarray’s shape is [2, 3] and the broadcast shape is [3, 2, 3]. Then the ndarray’s shape becomes [1, 2, 3] after padding and [3, 2, 3] after repeating the first axis.

Panics

This method panics if the target shape is incompatible with the ndarray.

  • If shape.len() is less than the dimensionality of the ndarray.
  • If a dimension in shape does not equal the corresponding dimension in the ndarray’s shape and cannot be broadcasted (i.e., it is not 1 or does not match).
Example
let ndarray = NdArray::new([1, 2, 3]);  // shape is [3]
let broadcasted_array = ndarray.broadcast_to(&[2, 3]);

assert_eq!(broadcasted_array.shape(), &[2, 3]);
Source

impl<T: NumericDataType> NdArray<'_, T>

pub fn astype<'b, F: NumericDataType>(&self) -> NdArray<'b, F>

Source

impl<'a, T: MatrixOps> NdArray<'a, T>

pub fn matmul<'r>(&self, other: impl AsRef<NdArray<'a, T>>) -> NdArray<'r, T>

Calculates the matrix product of two ndarrays.

  • If both ndarrays are 1D, then their dot product is returned.
  • If both ndarrays are 2D, then their matrix product is returned.
  • If the first ndarray is 2D and the second ndarray is 1D, then the matrix-vector product is returned.
Panics
  • If the dimensions/shape of the ndarrays are incompatible
Example

let a = NdArray::new(vec![
    [1, 2, 3],
    [4, 5, 6],
]);

let b = NdArray::new(vec![
    [7, 8],
    [9, 10],
    [11, 12],
]);

let result = a.matmul(&b);
assert_eq!(result, NdArray::new(vec![
    [58, 64],
    [139, 154],
]));
pub fn bmm<'r>(&self, other: impl AsRef<NdArray<'a, T>>) -> NdArray<'r, T>

Performs batch matrix multiplication on 3D ndarrays.

The shape of the resulting array will be [batch_size, self.shape()[1], other.shape()[2]], where batch_size is the shared first dimension of both input arrays.

Panics
  • If either array is not 3D
  • If the arrays do not have dimensions compatible for batch matrix multiplication.
Example

let arr1 = NdArray::<f32>::rand([3, 2, 4]); // 3 batches of 2x4 matrices
let arr2 = NdArray::<f32>::rand([3, 4, 5]); // 3 batches of 4x5 matrices
let result = arr1.bmm(&arr2);
assert_eq!(result.shape(), [3, 2, 5]); // result is 3 batches of 2x5 matrices
Source

impl<'a, T: SumOfProductsType> NdArray<'a, T>

pub fn dot<'b, 'r>(&self, other: impl AsRef<NdArray<'b, T>>) -> NdArray<'r, T>

Calculates the dot product of two 1D arrays.

Panics
  • Panics if either array is not 1D
  • Panics if the lengths of the two arrays are not equal
Examples
let arr1 = NdArray::new([1, 2, 3]);
let arr2 = NdArray::new([4, 5, 6]);
let result = arr1.dot(arr2);
assert_eq!(result.value(), 32); // 1*4 + 2*5 + 3*6 = 32
Source

impl<'a, T: NumericDataType> NdArray<'a, T>

pub fn trace<'r>(&self) -> NdArray<'r, T>

Returns the trace of the ndarray along its first 2 axes.

Panics
  • if the ndarray has fewer than 2 dimensions.
Examples
let arr = NdArray::new([
    [1, 2, 3],
    [4, 5, 6],
    [7, 8, 9]
]);

assert_eq!(arr.trace(), NdArray::scalar(1 + 5 + 9));
pub fn offset_trace<'r>(&self, offset: isize) -> NdArray<'r, T>

Returns the sum of an offset ndarray diagonal along its first 2 axes.

Panics
  • if the ndarray has fewer than 2 dimensions.
Examples
let arr = NdArray::new([
    [1, 2, 3],
    [4, 5, 6],
    [7, 8, 9]
]);

assert_eq!(arr.offset_trace(-1), NdArray::scalar(4 + 8));
pub fn trace_along<'r>( &self, axis1: impl AxisType, axis2: impl AxisType, ) -> NdArray<'r, T>

Returns the trace of an ndarray along the specified axes.

Panics
  • if the ndarray has fewer than 2 dimensions.
  • if axis1 and axis2 are the same or are out-of-bounds
Examples
let ndarray = NdArray::new([
    [1, 2, 3],
    [4, 5, 6],
    [7, 8, 9]
]);

assert_eq!(ndarray.trace_along(0, 1), NdArray::scalar(1 + 5 + 9));
pub fn offset_trace_along<'r>( &self, offset: isize, axis1: impl AxisType, axis2: impl AxisType, ) -> NdArray<'r, T>

Returns the sum of an offset ndarray diagonal along the specified axes.

Panics
  • if the ndarray has fewer than 2 dimensions.
  • if axis1 and axis2 are the same or are out-of-bounds
Examples
let ndarray = NdArray::new([
    [1, 2, 3],
    [4, 5, 6],
    [7, 8, 9]
]);

assert_eq!(ndarray.offset_trace_along(1, 0, 1), NdArray::scalar(2 + 6));
Source

impl<'a, T: RawDataType> NdArray<'a, T>

pub fn diagonal(&'a self) -> NdArray<'a, T>

Returns a diagonal view of the ndarray along its first 2 axes.

Panics
  • if the ndarray has fewer than 2 dimensions.
Examples
let ndarray = NdArray::new([
    [1, 2, 3],
    [4, 5, 6],
    [7, 8, 9]
]);

let diagonal = ndarray.diagonal();
assert_eq!(diagonal, NdArray::new([1, 5, 9]));
pub fn offset_diagonal(&'a self, offset: isize) -> NdArray<'a, T>

Returns an offset diagonal view of the ndarray along its first 2 axes.

Panics
  • if the ndarray has fewer than 2 dimensions.
Examples
let ndarray = NdArray::new([
    [1, 2, 3],
    [4, 5, 6],
    [7, 8, 9]
]);

let diagonal = ndarray.offset_diagonal(1);
assert_eq!(diagonal, NdArray::new([2, 6]));
pub fn diagonal_along( &'a self, axis1: impl AxisType, axis2: impl AxisType, ) -> NdArray<'a, T>

Returns a diagonal view of the ndarray along the specified axes.

Panics
  • if the ndarray has fewer than 2 dimensions.
  • if axis1 and axis2 are the same or are out-of-bounds
Examples
let ndarray = NdArray::new([
    [1, 2, 3],
    [4, 5, 6],
    [7, 8, 9]
]);

let diagonal = ndarray.diagonal_along(Axis(0), Axis(1));  // or .diagonal_along(0, 1)
assert_eq!(diagonal, NdArray::new([1, 5, 9]));
pub fn offset_diagonal_along( &'a self, offset: isize, axis1: impl AxisType, axis2: impl AxisType, ) -> NdArray<'a, T>

Returns an offset diagonal view of the ndarray along the specified axes.

Panics
  • if the ndarray has fewer than 2 dimensions.
  • if axis1 and axis2 are the same or are out-of-bounds
Examples
let arr = NdArray::new([
    [1, 2, 3],
    [4, 5, 6],
    [7, 8, 9]
]);

let diagonal = arr.offset_diagonal_along(-1, Axis(0), Axis(1));  // or .offset_diagonal_along(-1, 0, 1)
assert_eq!(diagonal, NdArray::new([4, 8]));