Tensor

Struct Tensor 

pub struct Tensor<'a, T: TensorDataType> { /* private fields */ }

Implementations

Source

impl<'a, T: TensorDataType> Tensor<'a, T>

pub fn value(&self) -> T

Retrieves the single value contained within a tensor with a singular element.

Panics

If the tensor contains more than one element (i.e., it is not a scalar or a tensor with a single element)

Example

let tensor = Tensor::scalar(50.0);
let value = tensor.value();
assert_eq!(value, 50.0);
Notes

This function is only meant for arrays that are guaranteed to have exactly one element. For arrays with multiple elements, consider using appropriate methods to access individual elements or slices safely.

pub fn ndarray(&self) -> &NdArray<'a, T>

Returns a reference to the underlying NdArray of the tensor

pub fn get_ndarray(&self) -> Rc<NdArray<'static, T>>

Returns a reference-counted pointer to the underlying NdArray of the tensor

pub fn into_ndarray(self) -> NdArray<'static, T>

Converts the tensor to an NdArray

Source

impl<'a, T: TensorDataType> Tensor<'a, T>

pub fn is_leaf(&self) -> bool

Checks if the tensor is a leaf.

A tensor is considered a leaf node if requires_grad = true and it was explicitly created by the user, or if requires_grad = false.

Examples

let mut tensor = Tensor::new([1.0, 2.0, 3.0]);
tensor.set_requires_grad(true);
assert!(tensor.is_leaf());

let tensor2 = -tensor;
assert!(!tensor2.is_leaf());
pub fn requires_grad(&self) -> bool

Returns whether gradients must be computed for this tensor.

A tensor is marked with the requires_grad flag if it was explicitly specified by the user through the set_requires_grad() method or if the tensor was created using operations on other tensors which were marked requires_grad.

Examples

let mut tensor = Tensor::new([1.0, 2.0, 3.0]);
tensor.set_requires_grad(true);

let tensor2 = -tensor;
assert!(tensor2.requires_grad());
pub fn set_requires_grad(&mut self, requires_grad: bool) -> &mut Self

Sets whether gradients must be computed for this tensor.

pub fn gradient(&'a self) -> Option<NdArray<'a, T>>

Returns the gradient of the differentiated tensor with respect to self.

This method returns a view into the gradient.

Examples

let mut a = Tensor::scalar(2.0f32);
let b = Tensor::scalar(3.0);

a.set_requires_grad(true);

let c = &a * &b;
c.backward();

// dc/da = b
assert_eq!(a.gradient().unwrap(), b);
pub fn zero_gradient(&self)

Sets the gradient of this tensor to zero.

Examples

let mut a = Tensor::scalar(2.0f32);
let b = Tensor::scalar(3.0);

a.set_requires_grad(true);

let c = &a * &b;
c.backward();

a.zero_gradient();
assert_eq!(a.gradient().unwrap(), Tensor::scalar(0.0));
pub fn backward_with(&self, gradient: impl AsRef<NdArray<'a, T>>)

Computes the gradient of the self with respect to its leaf tensors.

Parameters
  • gradient: the gradient of the tensor being differentiated with respect to self.
Examples

let mut a = Tensor::full(2.0, [3]);  // [2, 2, 2]
let b = Tensor::new([3.0, 1.0, -1.0]);

a.set_requires_grad(true);

let c = &a * &b;
c.backward_with(NdArray::new([2.0, 1.0, 1.0]));

// dc/da = b
assert_eq!(a.gradient().unwrap(), Tensor::new([6.0, 1.0, -1.0]));
pub fn backward(&self)

Computes the gradient of the self with respect to its leaf tensors.

Examples

let mut a = Tensor::full(2.0, [3]);  // [2, 2, 2]
let b = Tensor::new([3.0, 1.0, -1.0]);

a.set_requires_grad(true);

let c = &a * &b;
c.backward();

// dc/da = b
assert_eq!(a.gradient().unwrap(), Tensor::new([3.0, 1.0, -1.0]));
pub fn detach(&self) -> NdArray<'static, T>

Detaches the tensor from the computation graph and returns an NdArray.

Examples

let mut a = Tensor::full(2.0, [3]);
a.set_requires_grad(true);

let c = &a * 5.0;

let d = c.detach();
assert_eq!(d, NdArray::new([10.0, 10.0, 10.0]));
Source

impl<'a, T: TensorDataType> Tensor<'a, T>

pub fn dot<'b, 'r>(&self, other: impl AsRef<Tensor<'b, T>>) -> Tensor<'r, T>

Calculates the dot product of two 1D tensors.

Panics
  • Panics if either tensor is not 1D
  • Panics if the lengths of the two tensors are not equal
Examples
let tensor1 = Tensor::new([1.0, 2.0, 3.0]);
let tensor2 = Tensor::new([4.0, 5.0, 6.0]);
let result = tensor1.dot(tensor2);
assert_eq!(result.value(), 32.0); // 1*4 + 2*5 + 3*6 = 32
pub fn matmul<'r>(&self, other: impl AsRef<Tensor<'a, T>>) -> Tensor<'r, T>

Calculates the matrix product of two tensors.

  • If both tensors are 1D, then their dot product is returned.
  • If both tensors are 2D, then their matrix product is returned.
  • If the first tensor is 2D and the second tensor is 1D, then the matrix-vector product is returned.
Panics
  • If the dimensions/shape of the tensors are incompatible
Example

let a = Tensor::new(vec![
    [1.0, 2.0, 3.0],
    [4.0, 5.0, 6.0],
]);

let b = Tensor::new(vec![
    [7.0, 8.0],
    [9.0, 10.0],
    [11.0, 12.0],
]);

let result = a.matmul(&b);
assert_eq!(result, Tensor::new([
    [58.0, 64.0],
    [139.0, 154.0],
]));
pub fn bmm<'r>(&self, other: impl AsRef<Tensor<'a, T>>) -> Tensor<'r, T>

Performs batch matrix multiplication on 3D tensors.

The shape of the resulting ndarray will be [batch_size, self.shape()[1], other.shape()[2]], where batch_size is the shared first dimension of both input tensors.

Panics
  • If either tensor is not 3D
  • If the tensors do not have dimensions compatible for batch matrix multiplication.
Example

let arr1 = Tensor::<f32>::rand([3, 2, 4]); // 3 batches of 2x4 matrices
let arr2 = Tensor::<f32>::rand([3, 4, 5]); // 3 batches of 4x5 matrices
let result = arr1.bmm(&arr2);
assert_eq!(result.shape(), [3, 2, 5]); // result is 3 batches of 2x5 matrices